/* gcc integrator_gsl.c -o integrator_gsl -Wall -lgsl -lgslcblas -lm
 */

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<gsl/gsl_integration.h>

#define X_A 1
#define X_B 701
#define ACC 0.001
#define EPS 0.0

struct my_f_params { double I; double E; double T; };

double my_f (double x, void * p) {
   struct my_f_params * params 
     = (struct my_f_params *)p;
   double I = (params->I);
   double E = (params->E);
   double T = (params->T);

   return I * exp( 1 + (E/(8.617e-5*x))*((x-T)/T) - (x*x)/(T*T) * exp((E/(8.617e-5*x))*((x-T)/T)*(1-(E*8.617e-5*x/E)))-(E*8.617e-5*T/E));
}


int main (int argc, char *argv[]) {
  
  double err, err2, result;
  gsl_function F;
  struct my_f_params params = { 3.0, 2.0, 1.0 };
  gsl_integration_workspace *w  = gsl_integration_workspace_alloc (1000);
  




  FILE *fp;

  int a, b, max_peak_number;
  float temp0, temp1;
  float i[7], nerr[7], t[7], e[7];



  if ((argc != 5) && (argc != 2)) {
    printf(" error:  Too few arguments.\n");
    printf(" usage:  %s I E T error\n", argv[0]);
    exit(-1);
  }


  printf("PEAK# \t TEMP (K)\t INTENSITY \t  AREA \t\t +/- \n");



  if (argc == 5) {
    params.I =  atof(argv[1]);
    params.E =  atof(argv[2]);
    params.T =  atof(argv[3]);
    err2 = atof(argv[4]);

    F.function = &my_f;
    F.params = &params;
  
    gsl_integration_qags (&F, X_A, X_B, ACC, EPS, 1000,
			  w, &result, &err);
    //  printf("GSL QAG result : %f    Error: %e   Intervals: %i\n", result, err, w->size);
    printf(" N/A \t %.2f \t %.2f \t  %.2f \t %.2f\n", params.T, params.I, result, err2 * result);

    //    printf("TEMP : %f AREA : %f  +/- %f\n", t[a], result, err2 * result);
  }


  /* get parameters from function instead */
  if(argc == 2) {
    
    b = 1;  /* init b to something so we can enter the while loop */

    fp = fopen(argv[1],"r");
    /* skip first three lines */
    while ( getc(fp) != '\n' ) {}
    while ( getc(fp) != '\n' ) {}
    while ( getc(fp) != '\n' ) {}
    
    while ((b != EOF) && (b != 0)) {
      b = fscanf(fp, "I%i = %f +/- %f %*s\n", &a, &temp0, &temp1);
      i[a] = temp0; nerr[a] = temp1 / i[a];

      //   while ( getc(fp) != '\n' ) {}
      //      if (b == 3)
      // printf("%f %f\n", i[a], nerr[a]);

    }
    max_peak_number = a;

    // printf("max: %i  ", max_peak_number);

    a = 0;

    while (a < max_peak_number) {
      b = fscanf(fp, "FIXED: T%i = %f\n", &a, &temp0);
      t[a] = temp0;
      //if (b == 2)
      //      printf("%i %f\n", a, t[a]);
    }
  
    a = 0;
    while (a < max_peak_number) {
      b = fscanf(fp, "FIXED: E%i = %f\n", &a, &temp0);
      e[a] = temp0;
      //if (b == 2)
      //  printf("%i %f\n", a, e[a]);
    }
  


    if (a==0) {
      
      b = fscanf(fp, "FIXED: T%i = %f\n", &a, &temp0);
      t[a] = temp0;
      //if (b == 2)
      //      printf("%i %f\n", a, t[a]);

      b = fscanf(fp, "FIXED: E%i = %f\n", &a, &temp0);
      e[a] = temp0;
      //if (b == 2)
      //      printf("%i %f\n", a, e[a]);
    }


    

    fclose(fp);

    //  exit(0);
    
    for (a = 0; a <= max_peak_number; a++) {
      
      params.I =  i[a];
      params.E =  e[a];
      params.T =  t[a];
      err2 = nerr[a];
      
      F.function = &my_f;
      F.params = &params;            
      gsl_integration_qags (&F, X_A, X_B, ACC, EPS, 1000,
			    w, &result, &err);
      //  printf("GSL QAG result : %f    Error: %e   Intervals: %i\n", result, err, w->size);
      printf(" %i \t %.2f \t %.2f \t  %.2f \t %.2f\n", a, t[a], i[a], result, err2 * result);
    }
  }
  return(0);
}
